<?php
class eshoplogistic2
{

    public $modx;
	public $pdoTools;
	public $version = '0.0.1';
	
	public $payments;

    function __construct(modX &$modx, array $config = array())
    {
        $this->modx =& $modx;
		$this->pdoTools = $this->modx->getService('pdoTools');
		
		$this->payments = [
			'cash' => explode(',', $this->modx->getOption('eshoplogistic2_payment_cash')),
			'card' => explode(',', $this->modx->getOption('eshoplogistic2_payment_card')),
			'cashless' => explode(',', $this->modx->getOption('eshoplogistic2_payment_cashless')),
			'prepay' => explode(',', $this->modx->getOption('eshoplogistic2_payment_prepay'))
		];

        $corePath = $this->modx->getOption('eshoplogistic2_core_path', $config,
            $this->modx->getOption('core_path') . 'components/eshoplogistic2/'
        );
        $assetsUrl = $this->modx->getOption('eshoplogistic2_assets_url', $config,
            $this->modx->getOption('assets_url') . 'components/eshoplogistic2/'
        );
        $connectorUrl = $assetsUrl . 'connector.php';

        $this->config = array_merge(array(
			'actionUrl' => $assetsUrl . 'action.php',
            'assetsUrl' => $assetsUrl,
            'cssUrl' => $assetsUrl . 'css/',
            'jsUrl' => $assetsUrl . 'js/',
            'imagesUrl' => $assetsUrl . 'images/',
            'connectorUrl' => $connectorUrl,
            'corePath' => $corePath,
            'modelPath' => $corePath . 'model/',
            'chunksPath' => $corePath . 'elements/chunks/',
            'templatesPath' => $corePath . 'elements/templates/',
            'chunkSuffix' => '.chunk.tpl',
            'snippetsPath' => $corePath . 'elements/snippets/',
            'processorsPath' => $corePath . 'processors/',
        ), $config);

        $this->modx->addPackage('eshoplogistic2', $this->config['modelPath']);
        $this->modx->lexicon->load('eshoplogistic2:default');
    }
	
	
	
	
	public function initialize($ctx = 'web', array $scriptProperties = array()) 
	{
        if (isset($this->initialized[$ctx])) {
            return $this->initialized[$ctx];
        }

        $this->config = array_merge($this->config, $scriptProperties, array('ctx' => $ctx));

        if ($ctx != 'mgr' AND (!defined('MODX_API_MODE') OR !MODX_API_MODE)) {}
        
        $config = $this->pdoTools->makePlaceholders($this->config);

        $payments = [
            'card' => explode(',', $this->modx->getOption('eshoplogistic2_payment_card')),
            'cash' => explode(',', $this->modx->getOption('eshoplogistic2_payment_cash')),
            'cashless' => explode(',', $this->modx->getOption('eshoplogistic2_payment_cashless')),
            'prepay' => explode(',', $this->modx->getOption('eshoplogistic2_payment_prepay'))
        ];
        
        $data = json_encode([
            'actionUrl' => $this->config['actionUrl'],
            'assetsUrl' => $this->config['assetsUrl'],
            'payment_on' => $this->modx->getOption('eshoplogistic2_payment_on'),
            'payments' => json_encode($payments),
			'no_delivery_id' => $this->modx->getOption('eshoplogistic2_no_delivery_id'),
			'currency' => $this->modx->lexicon('eshoplogistic2_frontend_currency'),
            'price' => $this->modx->lexicon('eshoplogistic2_frontend_price'),
            'time' => $this->modx->lexicon('eshoplogistic2_frontend_time'),
            'select_city' => $this->modx->lexicon('eshoplogistic2_frontend_select_city'),
            'empty_pvz' => $this->modx->lexicon('eshoplogistic2_frontend_empty_pvz')
        ], true);
        $this->modx->regClientStartupScript('<script type="text/javascript">eshoplogistic2Config = '.$data.';</script>', true);
        
		$css = trim($this->modx->getOption('eshoplogistic2_frontend_css'));
        if (!empty($css) && preg_match('/\.css$/i', $css)) {
            $this->modx->regClientCSS(str_replace($config['pl'], $config['vl'], $css.'?v='.$this->version));
        }
        
        $js = trim($this->modx->getOption('eshoplogistic2_frontend_js'));
        if (!empty($js) && preg_match('/\.js/i', $js)) {
            $this->modx->regClientScript(str_replace($config['pl'], $config['vl'], $js.'?v='.$this->version)); //time())); 
        }
        
        $initialize = true;
        $this->initialized[$ctx] = $initialize;

        return $initialize;
    }
	
	

	
	public function Run($handler = 'delivery', $data = []) 
	{
        $class = ucfirst($handler);

		$file = dirname(__FILE__) . '/handlers/'.$handler.'.php';
		if(file_exists($file)) {
			require_once $file;
			$class = new $class($this);
			return $class->Process($data);
		}
		return false;
	}
	
	
	
	
	/*
	 * Вернёт: населённый пункт посетителя, используемые способы доставки esl, параметры заказа (offers)
	 */
	public function initCheckout($fias=null, $city=null)
	{
        unset($_SESSION['eshoplogistic2']);

        $out = [
            'widget_key' => $this->modx->getOption('eshoplogistic2_widget_key'),
		    'to' => [],
            'deliveries' => $_SESSION['eshoplogistic2']['deliveries'] ?? [],
            'offers' => $this->getOffers('cart')
        ];

        if(!empty($city) && !empty($_SESSION['eshoplogistic2']['to'][$city])) {
            $out['to'] = $_SESSION['eshoplogistic2']['to'][$city];
        } else {
            if($target = $this->setTarget($fias, $city)) {
                $out['to'] = $_SESSION['eshoplogistic2']['to'][$target['name']] = [
                    'name' => $target['target'],
                    'type' => $target['type'],
                    'fias' => $target['fias'],
                    'services' => $target['to']
                ];
            }
        }

        if(empty($_SESSION['eshoplogistic2']['deliveries'])) {
            $q = $this->modx->newQuery('msDelivery', ['active' => 1, 'class' => 'eslHandler']);
            foreach ($list = $this->modx->getIterator('msDelivery', $q) as $item) {
                $properties = json_decode($item->properties, 1);
                if (!empty($properties['mode'])) {
                    $out['deliveries'][$properties['mode']] = $item->id;
                }
            }

            $_SESSION['eshoplogistic2']['deliveries'] = $out['deliveries'];
        }


		return $out;
	}
	

	
	public function setTarget($fias = null, $city = null)
	{
	    if(!empty($fias)) {
			$search = [
				'fias' => $fias
			];
		}
		elseif(!empty($city)) {
			$search = [
				'city' => $city
			];
		}
		else {
			$search = [
				'ip' => $_SERVER['REMOTE_ADDR']
			];
		}
				
		$response = $this->Query('target', $search);

		if(!empty($response['data']['from']) && !empty($response['data']['to'])) {
		    return $response['data'];
		}

		false;
	}

	
	public function Query($method='init', $data=[]) 
	{			
		$apiKey = $this->modx->getOption('eshoplogistic2_api_key');
		if(empty($apiKey)) {
			$this->modx->log(1, 'eshoplogistic2: необходимо указать Ключ API');
			return [];
		}
		
		$curl = curl_init();
        curl_setopt($curl, CURLOPT_URL, 'https://api.eshoplogistic.ru/api/'.$method);
        curl_setopt($curl, CURLOPT_RETURNTRANSFER, 1);
        curl_setopt($curl, CURLOPT_TIMEOUT, 10);
        curl_setopt($curl, CURLOPT_POST, 1);
        curl_setopt($curl, CURLOPT_POSTFIELDS, array_merge($data,['key' => $apiKey]));
		$result = curl_exec($curl); 
	    curl_close($curl); 
	    
		return json_decode($result,1);
	}

	
	public function getOffers($mode, $product_id = false) 
	{
		$out = '';
		$offers = [];
		
		switch($mode) {
			case 'cart':
				$offers = $this->getOffersFromCart();
				break;
			case 'product':
				$offers = $this->getOfferAtProduct($product_id);
				break;
			case 'cart-or-product':
				$offers = $this->getOffersFromCart();
				if(empty($offers)) {
					$offers = $this->getOfferAtProduct($product_id);
				}
				break;
		}

		if (!empty($offers)) {

            $nfd = $this->modx->invokeEvent('eshoplogistic2OnGetOffers', [
                'offers' => $offers
            ]);

            if(!empty($nfd)) {
                if(is_array($nfd)) {
                    foreach($nfd as $odata) {
                        if(!empty($odata['new_offers'])) {
                            $offers = $odata['new_offers'];
                            break;
                        }
                    }
                }
            }
			
			$out = '[';
			foreach ($offers as $offer) {
				$out .= json_encode($offer) . ',';
			}
			$out = substr($out, 0, -1);
			$out .= ']';
			
			$s  = array('{', '}');
            $r = array('{ ', ' }');
    
            $out = str_replace($s, $r, $out);
		}

		
		return $out;
	}
	
	
	private function getOffersFromCart($for_widget = false)
	{	
		$out = [];
		if (!empty($_SESSION['minishop2']['cart'])) {
			foreach ($_SESSION['minishop2']['cart'] as $item) {

                $out[$item['id']] = [
                    'article' => $item['id'],
                    'name' => $item['id'],
                    'count' => $item['count'],
                    'price' => $item['price'],
                    'weight' => $item['weight']
                ];

			    if($for_widget) {
                    if ($product = $this->modx->getObject('msProduct', $item['id'])) {
                        $out[$item['id']]['name'] = $product->get('pagetitle');
                        $out[$item['id']]['image'] = $product->get('thumb');
                    }
                }
			}
		}
		return $out;
	}
		

	private function getOfferAtProduct($product_id) 
	{
		$offers = [];
		if($product_id) {
			if ($product = $this->modx->getObject('msProduct', $product_id)) {
				$offers[$product_id] = [
					'article' => $product_id,
					'name' => $product->get('pagetitle'),
					'count' => 1,
					'price' => $product->get('price'),
					'weight' => $product->get('weight'),
					'image' => $product->get('thumb')
				];
			}
		}
		return $offers;
	}

}