<?php

class Order 
{
	public $modx;
    public $eshoplogistic2;
	public $ms2;
	
	public $wDataSn = [
		'string' => ['path','comment', 'name','email','phone','deliveryTo','selectedDelivery','idShipper','addressPoint','selectedPayment','addressForDelivery'],
		'int' => ['costDelivery'],
		'double' => ['totalCost','weight'],
		'json' => ['city', 'offers','idShipper','selectedPayment','selectedDelivery']
	];
	
	public $wData = [];
	public $messages = [];
	public $deliveries = [];
		
    	
    function __construct($eshoplogistic2) 
	{
        $this->eshoplogistic2 = &$eshoplogistic2;
        $this->modx = &$eshoplogistic2->modx;
    }
	
	
	
	
	public function Process($data=[]) 
	{
		$this->messages = [
			$this->modx->getOption('eshoplogistic2_message_success'),
			$this->modx->getOption('eshoplogistic2_message_error')
		];
		
		
		if(!$this->checkSecret()){
			$this->modx->log(1, '[eshoplogistic2] access error: check the secret code of the widget');
			return $this->Response(false, 1);
		}		
		
		$this->acceptData();
		
		$dl = $this->modx->getIterator('msDelivery', ['active' => 1]);
		foreach($dl as $item) {
			$d_ = $item->toArray();
			if(!empty($d_['properties']['service'])) {
				$this->deliveries[$d_['properties']['mode']] = $d_['id'];
			}
		}
		
		return $this->createOrder();
	}
	
	
	
	
	
	private function checkSecret() 
	{
		return true;
		
		$secret = (string)$_POST['secret'] ?? '';
		
		if(!empty($secret)) {
			
			$keys = explode(',',trim($this->modx->getOption('eshoplogistic2_widget_keys')));

			if(is_array($keys)){
				foreach($keys as $key) {
					if($secret == $key) {
						return true;
					}
				}
			}
		}
		
		return false;
	}
	
	
	
	
	private function acceptData() 
	{
		foreach($this->wDataSn as $type => $items) {
			
			foreach($items as $item) {
				switch ($type) {
					case 'string':
						$this->wData[$item] = filter_input(INPUT_POST, $item, FILTER_SANITIZE_STRING);
						break;
					case 'int':
						$this->wData[$item] = filter_input(INPUT_POST, $item, FILTER_SANITIZE_NUMBER_INT);
						break;
					case 'double':
						$this->wData[$item] = (float)($_POST[$item]);
						break;
					case 'json':
						$this->wData[$item] = json_decode($_POST[$item],1);
						break;
				}
			}
			
		}
		#$this->modx->log(1,print_r($this->wData,1));
	}
	
	
	
	protected function createOrder()
	{		
		if($this->ms2Init()) {
			
			$this->ms2->order->clean();
			
			$order_num = trim($this->modx->getOption('eshoplogistic2_orderprefix')).$this->ms2->order->getNum();

			$fields = ['receiver' => '', 'email' => '', 'phone' => '', 'comment' => ''];
				
			foreach($fields as $k => $v) {
				if($k == 'receiver' && !empty($this->wData['name'])) {
					$fields[$k] = $this->wData['name'];
				}
				else {
					if(!empty($this->wData[$k])) {
						$fields[$k] = $this->wData[$k];
					}
				}
				$this->ms2->order->add($k, $fields[$k]); 
			}
			
			
            if(!empty($this->wData['addressForDelivery'])) {
                
                $fields['comment'] .= "\nПолный адрес доставки: ".$this->wData['addressForDelivery'];
                
                if($address = explode(',', $this->wData['addressForDelivery'])) {
                    $fields['index'] = $address[0];
                    $fields['city'] = $address[1] ?? '';
                    $fields['street'] = $address[2] ?? '';
                    $fields['room'] = $address[3] ?? '';
                }
            }
            
            if(!empty($this->wData['city']['region'])) {
                $fields['region'] = $this->wData['city']['region'];
            }
			 if(!empty($this->wData['city']['name'])) {
                $fields['city'] = $this->wData['city']['name'];
            }          
            
            $properties = [
                'esl' => [
                    'price' => $this->wData['costDelivery'] ?? 0,
                    'time' => $this->wData['selectedDelivery']['time'] ?? '',
                    'mode' => $this->wData['selectedDelivery']['name'] ?? '',
                    'service' => $this->wData['idShipper']['name'] ?? '',
                    'address' => ($this->wData['selectedDelivery']['key'] == 'terminal') ? $this->wData['addressForDelivery'] : ''
                ]  
            ];
				

			#$this->modx->log(1, print_r($this->wData,1));
			#$this->modx->log(1, print_r($this->deliveries,1));
			#$this->modx->log(1, print_r($this->eshoplogistic2->payments,1));
		
			
			$payment = (!empty($this->eshoplogistic2->payments[$this->wData['selectedPayment']['key']][0])) ?
				$this->eshoplogistic2->payments[$this->wData['selectedPayment']['key']][0] : 0;

			foreach($this->eshoplogistic2->payments as $k => $v) {
				if(!is_array($v)) {
					foreach($v as $pitem) {
						if($pitem == $this->wData['selectedPayment']['key']) {
							$payment = $k;
							break(2);
						}
					}
				}
			}
			
			$orderData = [
				'user_id' => $this->ms2->getCustomerId(),
				'delivery_cost' => $this->wData['costDelivery'],
				'createdon' => date('Y-m-d H:i:s'),
				'num' => $this->modx->getOption('eshoplogistic2_orderprefix').$this->ms2->order->getNum(),
				'delivery' => $this->deliveries[$this->wData['selectedDelivery']['key']] ?? 0,
				'payment' => $payment,
				'status' => 1,
				'properties' => $properties,
				'context' => $this->ms2->config['ctx'],
				'weight' => $this->wData['weight'],
				'cost' => 0,
				'cart_cost' => 0
			];

			if($this->wData['offers']) {

				foreach ($this->wData['offers'] as $product) {
					$orderData['cart_cost'] += $product['price'] * $product['count'];
				}
				
				$orderData['cost'] = $orderData['cart_cost'] + $this->wData['costDelivery'];

				$order = $this->modx->newObject('msOrder');
				$order->fromArray($orderData);

				$orderProducts = [];
				foreach ($this->wData['offers'] as $product) {
					$orderProduct = $this->modx->newObject('msOrderProduct');
					$orderProduct->fromArray([
						'product_id' => $product['article'],
						'name' => $product['name'],
						'count' => $product['count'],
						'price' => $product['price'],
						'weight' => $product['weight'],
						'cost' => $product['price'] * $product['count']
						//'options' => $item['options']
					]);
					$orderProducts[] = $orderProduct;
				}
				$order->addMany($orderProducts);
				
				$response = $this->ms2->invokeEvent('msOnBeforeCreateOrder', array(
					'msOrder' => $order,
					'order' => $this->ms2->order
				));
				
				if (!$response['success']) {
					$this->modx->log(1, '[eshoplogistic2] minishop2 msOnBeforeCreateOrder error: '.$response['message']);
					return $this->Response(false, 1);
				}

				$address = $this->modx->newObject('msOrderAddress');
				$address->fromArray(array_merge(['createdon' => date('Y-m-d H:i:s')], $fields));
				$order->addOne($address);

				if ($order->save()) {
					$this->ms2->invokeEvent('msOnCreateOrder', [
						'msOrder' => $order,
						'order' => $this->ms2->order
					]);
					$this->ms2->order->clean();
					$this->ms2->changeOrderStatus($order->get('id'), 1);

					return $this->Response(true, 0, $order_num);
				}
			}
		}
		else {
			$this->modx->log(1, '[eshoplogistic2] minishop2 initialization error');
			return $this->Response(false, 1);
		}
	}
	
	
	
	
	
	protected function ms2Init()
	{
		if(is_dir($this->modx->getOption('core_path').'components/minishop2/model/minishop2/')) {
			
			$this->ms2 = $this->modx->getService('miniShop2');
			
			if ($this->ms2 instanceof miniShop2) {
				
				$context = $this->modx->context->key ? $this->modx->context->key : 'web';
				
				$this->ms2->initialize($context, ['json_response' => true]);
				
				return true;
			}
		}
		
		return false;
	}
	
	
	
	
	
	protected function Response($success, $msg, $order_num=0)
	{
		$message = $this->messages[$msg];
		
		if(!empty($order_num)) {
			$message = str_replace('{num}',$order_num,$message);
		}
				
		return ['success' => $success, 'message' => $message];
	}
	
}

